"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const RequestMethod_1 = require("../../enums/RequestMethod");
const JSHelperUtil_1 = require("../../util/JSHelperUtil");
const JsonProtocol_1 = require("../../protocol/JsonProtocol");
const HttpRequestError_1 = require("../../error/HttpRequestError");
const HttpRequestErrorEnum_1 = require("../../enums/HttpRequestErrorEnum");
/**
 *
 * 功能描述:
 *
 * @className RestConnection
 * @projectName papio
 * @author yanshaowen
 * @date 2019/2/1 13:32
 */
class AxiosConnection {
    constructor(options) {
        this.readonlyConnection = false;
        this.options = options;
    }
    close() {
        return undefined;
    }
    commit(savePoint) {
        return undefined;
    }
    connect() {
        return undefined;
    }
    isClosed() {
        return false;
    }
    isConnected() {
        return true;
    }
    isReadOnly() {
        return this.readonlyConnection;
    }
    rollback(savePoint) {
        return undefined;
    }
    setReadOnly(readOnly) {
        this.readonlyConnection = readOnly;
    }
    setSavepoint(name) {
        return undefined;
    }
    startTransaction(level) {
        return undefined;
    }
    async request(result, genericsProperty, uri, method, timeout, params, body, headers) {
        if (!method) {
            method = RequestMethod_1.RequestMethod.GET;
        }
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(timeout)) {
            timeout = 0;
        }
        try {
            const Axios = require("axios");
            const response = await Axios.request({
                url: uri,
                baseURL: this.options.url,
                method: method,
                params: params,
                data: body,
                timeout: timeout,
                httpAgent: this.options.agent,
                headers: headers
            });
            if (response.status === 200) {
                return JsonProtocol_1.JsonProtocol.jsonToBean(response.data, result, genericsProperty);
            }
            else {
                const error = new HttpRequestError_1.HttpRequestError();
                error.code = HttpRequestErrorEnum_1.HttpRequestErrorEnum.STATUS_ERROR;
                error.message = 'status not equal to ' + response.status;
                throw error;
            }
        }
        catch (e) {
            const error = new HttpRequestError_1.HttpRequestError();
            error.code = e.code;
            error.message = e.message;
            if (e.response) {
                // 请求已发出，但服务器响应的状态码不在 2xx 范围内
                error.status = e.response.status;
                error.data = e.response.data;
            }
            throw error;
        }
    }
    static async build(options, isReadOnly) {
        const httpConnection = new AxiosConnection(options);
        httpConnection.setReadOnly(isReadOnly);
        return httpConnection;
    }
}
exports.AxiosConnection = AxiosConnection;
//# sourceMappingURL=AxiosConnection.js.map