"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 *
 * 功能描述: Json 转换协议 识别JsonProperty装饰器
 *
 * @className JsonProtocol
 * @projectName papio
 * @author yanshaowen
 * @date 2018/12/30 17:48
 */
require("reflect-metadata");
const MetaConstant_1 = require("../constants/MetaConstant");
const JSHelperUtil_1 = require("../util/JSHelperUtil");
const StringUtil_1 = require("../util/StringUtil");
const CommonConstant_1 = require("../constants/CommonConstant");
const DateUtil_1 = require("../util/DateUtil");
const DateFormatEnum_1 = require("../enums/DateFormatEnum");
class JsonProtocol {
    /**
     * 方法描述: bean 数组转json对象
     * @author  yanshaowen
     * @date 2018/12/30 17:50
     * @param beans 数组
     * @param beanGenericsMap
     * @param parentKey
     * @return JSON
     */
    static toArray(beans, beanGenericsMap, parentKey) {
        let _beanGenericsMap = new Map();
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(beanGenericsMap)) {
            _beanGenericsMap = new Map();
        }
        else {
            for (const [key, value] of beanGenericsMap) {
                _beanGenericsMap.set(key, value);
            }
        }
        return JsonProtocol._toArray(beans, _beanGenericsMap, parentKey);
    }
    static _toArray(beans, beanGenericsMap, parentKey) {
        const result = [];
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(beans)) {
            return result;
        }
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(parentKey)) {
            parentKey = beans.constructor.name;
        }
        // 检查泛型是否定义了
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(beanGenericsMap.get(parentKey))) {
            return result;
        }
        for (const bean of beans) {
            let newBean;
            if (JSHelperUtil_1.JSHelperUtil.isBaseType(beanGenericsMap.get(parentKey))) {
                // 基础类型的泛型 则直接赋值
                if (beanGenericsMap.get(parentKey) === Number) {
                    newBean = Number(bean);
                }
                else if (beanGenericsMap.get(parentKey) === String) {
                    newBean = String(bean);
                }
                else if (beanGenericsMap.get(parentKey) === Boolean) {
                    newBean = Boolean(bean);
                }
                else {
                    newBean = null;
                }
            }
            else {
                newBean = JsonProtocol._toJson(bean, beanGenericsMap, parentKey + "." + beanGenericsMap.get(parentKey).name);
            }
            result.push(newBean);
        }
        return result;
    }
    /**
     * 方法描述: bean 对象转json对象
     * @author  yanshaowen
     * @date 2018/12/30 17:50
     * @param bean 对象
     * @param beanGenericsMap   泛型字典
     * @param parentKey
     * @return JSON
     */
    static toJson(bean, beanGenericsMap, parentKey) {
        let _beanGenericsMap = new Map();
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(beanGenericsMap)) {
            _beanGenericsMap = new Map();
        }
        else {
            for (const [key, value] of beanGenericsMap) {
                _beanGenericsMap.set(key, value);
            }
        }
        return JsonProtocol._toJson(bean, _beanGenericsMap, parentKey);
    }
    static _toJson(bean, beanGenericsMap, parentKey) {
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(bean)) {
            return {};
        }
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(parentKey)) {
            parentKey = bean.constructor.name;
        }
        const result = {};
        const keysMap = Reflect.getOwnMetadata(MetaConstant_1.MetaConstant.KEYS, bean.constructor.prototype) || new Set();
        for (const key of keysMap) {
            let jsonOption = Reflect.getMetadata(MetaConstant_1.MetaConstant.JSON_PROPERTY, bean, key);
            const returnGenerics = Reflect.getOwnMetadata(MetaConstant_1.MetaConstant.BEAN_RETURN_GENERICS, bean.constructor.prototype, key) || new Map();
            let jsonKeyName = null;
            let jsonFormat = null;
            if (JSHelperUtil_1.JSHelperUtil.isNotNull(jsonOption)) {
                jsonKeyName = jsonOption.value;
                jsonFormat = jsonOption.format;
            }
            let typeName = Reflect.getMetadata(MetaConstant_1.MetaConstant.DESIGN_TYPE, bean, key);
            // const GenericsKey = Reflect.getMetadata(MetaConstant.BEAN_GENERICS, bean, key);
            const genericsKey = parentKey + "." + key;
            for (const [genKey, genValue] of returnGenerics) {
                if (!beanGenericsMap.has(genericsKey + "." + genKey)) {
                    beanGenericsMap.set(genericsKey + "." + genKey, genValue);
                }
            }
            if (StringUtil_1.StringUtil.isBank(jsonKeyName)) {
                jsonKeyName = key;
            }
            // 可能为泛型或者any
            if (typeName === Object) {
                // 判断是否有泛型字典
                if (beanGenericsMap.has(genericsKey) && JSHelperUtil_1.JSHelperUtil.isNotNull(beanGenericsMap.get(genericsKey))) {
                    typeName = beanGenericsMap.get(genericsKey);
                }
                else {
                    result[jsonKeyName] = null;
                    continue;
                }
            }
            if (typeName === Date && bean[key] instanceof Date) {
                if (JSHelperUtil_1.JSHelperUtil.isNotNull(jsonFormat)) {
                    result[jsonKeyName] = DateUtil_1.DateUtil.format(bean[key], jsonFormat);
                }
                else {
                    result[jsonKeyName] = DateUtil_1.DateUtil.format(bean[key], DateFormatEnum_1.DateFormatEnum.DATETIMES);
                }
            }
            else if (JSHelperUtil_1.JSHelperUtil.isClassType(typeName)) {
                result[jsonKeyName] = JsonProtocol._toJson(bean[key], beanGenericsMap, genericsKey + "." + typeName.name);
            }
            else if (JSHelperUtil_1.JSHelperUtil.isArrayType(typeName) || JSHelperUtil_1.JSHelperUtil.isSetType(typeName)) {
                result[jsonKeyName] = JsonProtocol._toArray(bean[key], beanGenericsMap, genericsKey + "." + (JSHelperUtil_1.JSHelperUtil.isArrayType(typeName) ? "Array" : "Set"));
                // array set
            }
            else {
                if (bean[key] === undefined) {
                    bean[key] = null;
                }
                result[jsonKeyName] = bean[key];
            }
        }
        return result;
    }
    /**
     * 方法描述: bean 对象转json字符串
     * @author  yanshaowen
     * @date 2018/12/30 17:50
     * @param bean 对象
     * @param beanGenericsMap   bean对象泛型字典
     * @param parentKey         父级key
     * @return JSON
     */
    static toJSONString(bean, beanGenericsMap, parentKey) {
        let json = {};
        if (Array.isArray(bean)) {
            json = JsonProtocol.toArray(bean, beanGenericsMap, parentKey);
        }
        else {
            json = JsonProtocol.toJson(bean, beanGenericsMap, parentKey);
        }
        return JSON.stringify(json);
    }
    /**
     * 方法描述: JSON数组转bean集合
     * @author  yanshaowen
     * @date 2018/12/30 17:50
     * @param array  源jsonArray对象
     * @param Bean  目标bean对象
     * @param beanGenericsMap
     * @param parentKey
     * @return Bean[]
     */
    static arrayToBeans(array, Bean, beanGenericsMap, parentKey) {
        let _beanGenericsMap = new Map();
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(beanGenericsMap)) {
            _beanGenericsMap = new Map();
        }
        else {
            for (const [key, value] of beanGenericsMap) {
                _beanGenericsMap.set(key, value);
            }
        }
        return JsonProtocol._arrayToBeans(array, Bean, _beanGenericsMap, parentKey);
    }
    static _arrayToBeans(array, Bean, beanGenericsMap, parentKey) {
        let result = [];
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(array)) {
            return null;
        }
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(Bean)) {
            return [];
        }
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(parentKey)) {
            parentKey = Bean.name;
        }
        // 检查泛型是否定义了
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(beanGenericsMap.get(parentKey))) {
            return result;
        }
        for (const bean of array) {
            let newBean;
            if (JSHelperUtil_1.JSHelperUtil.isBaseType(beanGenericsMap.get(parentKey))) {
                // 基础类型的泛型 则直接赋值
                if (beanGenericsMap.get(parentKey) === Number) {
                    newBean = Number(bean);
                }
                else if (beanGenericsMap.get(parentKey) === String) {
                    newBean = String(bean);
                }
                else if (beanGenericsMap.get(parentKey) === Boolean) {
                    newBean = Boolean(bean);
                }
                else {
                    newBean = null;
                }
            }
            else {
                newBean = JsonProtocol._jsonToBean(bean, beanGenericsMap.get(parentKey), beanGenericsMap, parentKey + "." + beanGenericsMap.get(parentKey).name);
            }
            result.push(newBean);
        }
        return result;
    }
    /**
     * 方法描述: JSON对象转bean对象
     * @author  yanshaowen
     * @date 2018/12/30 17:50
     * @param json  源json对象
     * @param Bean  目标bean对象
     * @param beanGenericsMap
     * @param parentKey
     * @return Bean
     */
    static jsonToBean(json, Bean, beanGenericsMap, parentKey) {
        let _beanGenericsMap = new Map();
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(beanGenericsMap)) {
            _beanGenericsMap = new Map();
        }
        else {
            for (const [key, value] of beanGenericsMap) {
                _beanGenericsMap.set(key, value);
            }
        }
        return JsonProtocol._jsonToBean(json, Bean, _beanGenericsMap, parentKey);
    }
    static _jsonToBean(json, Bean, beanGenericsMap, parentKey) {
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(Bean)) {
            return null;
        }
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(json)) {
            return new Bean();
        }
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(parentKey)) {
            parentKey = Bean.name;
        }
        const result = new Bean();
        // 遍历bean所有的属性
        const keysMap = Reflect.getOwnMetadata(MetaConstant_1.MetaConstant.KEYS, Bean.prototype) || new Set();
        for (const key of keysMap) {
            let jsonOption = Reflect.getMetadata(MetaConstant_1.MetaConstant.JSON_PROPERTY, Bean.prototype, key);
            const returnGenerics = Reflect.getOwnMetadata(MetaConstant_1.MetaConstant.BEAN_RETURN_GENERICS, Bean.prototype, key) || new Map();
            let jsonKeyName = null;
            let jsonFormat = null;
            if (JSHelperUtil_1.JSHelperUtil.isNotNull(jsonOption)) {
                jsonKeyName = jsonOption.value;
                jsonFormat = jsonOption.format;
            }
            let typeName = Reflect.getMetadata(MetaConstant_1.MetaConstant.DESIGN_TYPE, Bean.prototype, key);
            // const GenericsIndex = Reflect.getMetadata(MetaConstant.BEAN_GENERICS, Bean.prototype, key);
            const genericsKey = parentKey + "." + key;
            if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(jsonKeyName)) {
                jsonKeyName = key;
            }
            for (const [genKey, genValue] of returnGenerics) {
                if (!beanGenericsMap.has(genericsKey + "." + genKey)) {
                    beanGenericsMap.set(genericsKey + "." + genKey, genValue);
                }
            }
            // 可能为泛型或者any
            if (typeName === Object) {
                // 判断是否有泛型字典
                if (beanGenericsMap.has(genericsKey) && JSHelperUtil_1.JSHelperUtil.isNotNull(beanGenericsMap.get(genericsKey))) {
                    typeName = beanGenericsMap.get(genericsKey);
                }
                else {
                    result[key] = null;
                    continue;
                }
            }
            if (json[jsonKeyName] === undefined || json[jsonKeyName] === null) {
                result[key] = null;
            }
            else if (JSHelperUtil_1.JSHelperUtil.isBaseType(typeName)) {
                if (typeof json[jsonKeyName] === 'object') {
                    if (JSHelperUtil_1.JSHelperUtil.isStringType(typeName)) {
                        result[key] = JSON.stringify(json[jsonKeyName]);
                    }
                    else {
                        result[key] = null;
                    }
                }
                else {
                    if (JSHelperUtil_1.JSHelperUtil.isStringType(typeName)) {
                        result[key] = String(json[jsonKeyName]);
                    }
                    else if (JSHelperUtil_1.JSHelperUtil.isNumberType(typeName)) {
                        result[key] = Number(json[jsonKeyName]);
                    }
                    else if (JSHelperUtil_1.JSHelperUtil.isBooleanType(typeName)) {
                        result[key] = Boolean(json[jsonKeyName]);
                    }
                }
            }
            else if (typeName === Date && typeof json[jsonKeyName] === "string") {
                if (JSHelperUtil_1.JSHelperUtil.isNotNull(jsonFormat)) {
                    result[jsonKeyName] = DateUtil_1.DateUtil.parse(json[jsonKeyName], jsonFormat);
                }
                else {
                    result[jsonKeyName] = DateUtil_1.DateUtil.parse(json[jsonKeyName], DateFormatEnum_1.DateFormatEnum.DATETIMES);
                }
            }
            else if (JSHelperUtil_1.JSHelperUtil.isClassType(typeName)) {
                result[key] = JsonProtocol._jsonToBean(json[jsonKeyName], typeName, beanGenericsMap, genericsKey + "." + typeName.name);
            }
            else if (JSHelperUtil_1.JSHelperUtil.isArrayType(typeName) || JSHelperUtil_1.JSHelperUtil.isSetType(typeName)) {
                // array set
                result[key] = JsonProtocol._arrayToBeans(json[jsonKeyName], typeName, beanGenericsMap, genericsKey + "." + (JSHelperUtil_1.JSHelperUtil.isArrayType(typeName) ? "Array" : "Set"));
            }
            else {
                // type error
            }
        }
        return result;
    }
    /**
     * 方法功能描述: 根据返回值转换成bean对象
     * @author yanshaowen
     * @date 2019/2/19 10:18
     * @param value             转换的值
     * @param genericsProperty
     * @return
     */
    static routerToBean(value, genericsProperty) {
        const genRoot = genericsProperty.get(CommonConstant_1.CommonConstant.GENERICS_ROOT);
        if (JSHelperUtil_1.JSHelperUtil.isNullOrUndefined(value)) {
            return null;
        }
        else if (JSHelperUtil_1.JSHelperUtil.isBaseObject(value) && JSHelperUtil_1.JSHelperUtil.isBaseType(genRoot)) {
            // 基础类型的泛型 则直接赋值
            if (genRoot === Number) {
                return Number(value);
            }
            else if (genRoot === String) {
                return String(value);
            }
            else if (genRoot === Boolean) {
                return Boolean(value);
            }
            else {
                return null;
            }
        }
        else if ((typeof value === "object" || typeof value === "string") && (JSHelperUtil_1.JSHelperUtil.isClassType(genRoot) || Array === genRoot)) {
            let jsonParse;
            if (typeof value === "string") {
                try {
                    jsonParse = JSON.parse(value);
                }
                catch (e) {
                    throw new Error(`json parse error(${e.message},result(${value})`);
                }
            }
            else {
                jsonParse = value;
            }
            if (genRoot === Array) {
                return JsonProtocol.arrayToBeans(jsonParse, Array, genericsProperty, "Array");
            }
            else {
                return JsonProtocol.jsonToBean(jsonParse, genRoot, genericsProperty);
            }
        }
        else {
            throw new Error(`args ${value} error or genericsProperty error`);
        }
    }
    /**
     * 方法功能描述: 深拷贝bean对象
     * @author yanshaowen
     * @date 2019/2/22 9:25
     * @param sourceBean    源
     * @param targetBean    目标
     * @param converters    转换器
     * @return
     */
    static copyProperties(sourceBean, targetBean, converters) {
        if (!JSHelperUtil_1.JSHelperUtil.isClassObject(sourceBean) || !JSHelperUtil_1.JSHelperUtil.isClassObject(targetBean)) {
            throw new Error(`copy error,only support class to class.`);
        }
        // 加载converters
        const convertersMap = new Map();
        if (converters) {
            converters.forEach((v, i) => {
                if (v.clazz && v.keys) {
                    v.keys.forEach((key) => {
                        convertersMap.set(key, i);
                    });
                }
            });
        }
        // source bean所有的属性
        const sourceKeys = Reflect.getOwnMetadata(MetaConstant_1.MetaConstant.KEYS, sourceBean.constructor.prototype) || new Set();
        // target bean所有的属性
        const targetKeys = Reflect.getOwnMetadata(MetaConstant_1.MetaConstant.KEYS, targetBean.constructor.prototype) || new Set();
        for (const key of sourceKeys) {
            if (targetKeys.has(key)) {
                const sTypeName = Reflect.getMetadata(MetaConstant_1.MetaConstant.DESIGN_TYPE, sourceBean.constructor.prototype, key);
                const tTypeName = Reflect.getMetadata(MetaConstant_1.MetaConstant.DESIGN_TYPE, targetBean.constructor.prototype, key);
                // 先检查converter
                if (convertersMap.has(key)) {
                    const iConverter = converters[convertersMap.get(key)];
                    const clazz = iConverter.clazz;
                    if (clazz && clazz === sTypeName) {
                        targetBean[key] = iConverter.convert(key, sourceBean[key]);
                        continue;
                    }
                }
                if (JSHelperUtil_1.JSHelperUtil.isBaseType(sTypeName)) {
                    if (sTypeName === tTypeName) {
                        targetBean[key] = sourceBean[key];
                    }
                    else {
                        // 强制
                        try {
                            targetBean[key] = tTypeName(sourceBean[key]);
                        }
                        catch (e) {
                        }
                    }
                }
            }
        }
    }
}
exports.JsonProtocol = JsonProtocol;
//# sourceMappingURL=JsonProtocol.js.map