export declare class DateUtil {
    /**
     * 方法功能描述: 对比time1 比time2时间的大小  大于1 等于0 小于 -1
     * @author yanshaowen
     * @date 2019/2/23 10:48
     * @param time1
     * @param time2
     * @return
     */
    static compareTime(time1: string, time2: string): 1 | 0 | -1;
    /**
     * 方法功能描述: 是否闰年
     * @author yanshaowen
     * @date 2019/2/23 10:47
     * @param year
     * @return
     */
    static isLeapYear(year: number): boolean;
    /**
     * 方法功能描述: 获取某个月的天数，从0开始
     * @author yanshaowen
     * @date 2019/2/23 10:45
     * @param year  年
     * @param month 月
     * @return
     */
    static getDaysOfMonth(year: number, month: number): number;
    /**
     * 方法功能描述: 距离现在几天的日期：负数表示今天之前的日期，0表示今天，整数表示未来的日期
     * 如-1表示昨天的日期，0表示今天，2表示后天
     * @author yanshaowen
     * @date 2019/2/23 10:41
     * @param days  天数
     * @return      Date对象
     */
    static fromToday(days: number): Date;
    /**
     * 方法功能描述:
     * @author yanshaowen
     * @date 2019/2/23 10:40
     * @param date  需要格式化的日期对象
     * @param pattern 格式化的模式，如yyyy-MM-dd hh(HH):mm:ss.S a k K E D F w W z Z
     * @return 格式化后的时间
     */
    static format(date: Date, pattern: string): string;
    /**
     * 将字符串转换成日期
     * @param {string} datestr 日期时间字符串
     * @param {string} format 格式化字符串
     * yyyy:年份(如2019);
     * yy:短年份(如19);
     * MM:月份;
     * dd:日子;
     * HH:小时(0~23);
     * hh:小时(0~11);
     * mm:分钟;
     * ss:秒;
     * S:毫秒;
     * @returns {Date} 日期时间对象
     */
    static parse(datestr: string, format?: string): Date;
    /**
     * 方法功能描述: 计算一个日期是当年的第几天
     * @author yanshaowen
     * @date 2019/2/23 10:03
     * @param date
     * @return
     */
    static dayOfTheYear(date: any): number;
    /**
     * 方法功能描述: 获得时区名和值
     * @author yanshaowen
     * @date 2019/2/23 10:50
     * @param date
     * @return
     */
    static getZoneNameValue(date: Date): {
        name: string;
        value: string;
    };
}
